/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Some functions that didn't fit elsewhere */

#ifndef DEFINES_H
#define DEFINES_H
#include <defines.h>
#endif

#include "extra.h"
#include <xmalloc.h>
#include <lcrypt.h>
#include <bzero.h>
#include <bits.h>
#include "locks.h"
#include <errors.h>
#include <getpass.h>
#include <keys.h>


extern char *outfile;
extern int stream_flag;
extern int cleanDelete;

char *
 my_getpass(char *prt)
{
	char *atmp;
	char ztmp[MAX_KEY_LEN];
	char *btmp;

	atmp = getpass(prt);

	Bzero(ztmp, sizeof(ztmp));
	strncat(ztmp, atmp, MAX_KEY_LEN - 1);

	btmp = getpass(_("Re-Enter key: "));

	if (strcmp(ztmp, btmp) != 0) {
		fprintf(stderr, _("Keywords do not match or they are too long.\n"));
		return NULL;
	}
	return btmp;
}

char *
 get_password(int numofchar, int bitmode, int mode, int type, unsigned int *len, void *salt)
{

	char *tmp = NULL;
	char msg[100];

	switch (bitmode) {

	case HASH:
		sprintf(msg, _("Enter passphrase (max %d chars): "), MAX_KEY_LEN-1);

		if (mode == ENCRYPT) {
			tmp = my_getpass(msg);
		} else {
			tmp = getpass(msg);
		}
		if (tmp == NULL)
			return NULL;

		*len = get_key_size(type);
		gen_key(tmp, salt, SALT_SIZE, *len, (void *) tmp);
		break;


	case FULL:
		sprintf(msg, _("Enter key (%d chars): "), numofchar);

		if (mode == ENCRYPT) {
			tmp = my_getpass(msg);
		} else {
			tmp = getpass(msg);
		}
		if (tmp == NULL)
			return NULL;

		if ((strlen(tmp) < numofchar) && (type != BLOWFISH)) {
			fprintf(stderr, _("Warning: keyword is too small for this algorithm\n"));
		}
		if (strlen(tmp) > numofchar) {
			fprintf(stderr, _("Warning: Reducing the size of keyword\n"));
		}
		*len = strlen(tmp);
		break;


	case HALF:
		sprintf(msg, _("Enter key (%d chars): "), numofchar * 2);

		if (mode == ENCRYPT) {
			tmp = my_getpass(msg);
		} else {
			tmp = getpass(msg);
		}
		if (tmp == NULL)
			return NULL;

		if ((strlen(tmp) < numofchar * 2) && (type != BLOWFISH)) {
			fprintf(stderr, _("Warning: keyword is too small for this algorithm\n"));
		}
		*len = strip4bits(tmp, strlen(tmp));
		break;

	}

	return tmp;

}


int show_mode(int ende, int stype, int smode, char *output, int bitmode)
{
	char *x = get_algorithms_name(stype);

	fprintf(stderr, _("Algorithm: %s\n"), x);
	free(x);

	if (stype != CRYPT) {
		x = get_modes_name(smode);
		fprintf(stderr, _("Mode: %s\n"), x);
		free(x);
	}
	switch (bitmode) {
	case HASH:
		fprintf(stderr, _("Keyword Mode: SHA-1 HASH\n"));
		break;
	case FULL:
		fprintf(stderr, _("Keyword Mode: 8bits/char\n"));
		break;
	case HALF:
		fprintf(stderr, _("Keyword Mode: 4bits/char\n"));
		break;
	}

	if (ende == ENCRYPT) {
		fprintf(stderr, _("Crypting to %s ...\n"), output);
	} else {
		fprintf(stderr, _("Decrypting to %s ...\n"), output);
	}

	return 0;
}


#ifdef HAVE_STAT

int check_file(char *filename)
{
	struct stat ostat;

	if (stat(filename, &ostat) == 0) {
		return 1;	/* exists */
	} else {
		return 0;	/* no */
	}


}


#endif


int check_file_head(char *filename, unsigned int *algorithm, unsigned int *mode, unsigned int *bit_flag, unsigned int stream_flag)
{
	FILE *fstream;
	char buf[6];


	if (stream_flag == TRUE) {
		fstream = (FILE *) stdin;
	} else {
		fstream = fopen(filename, "r");
		if (read_lock(fileno((FILE *) fstream)) == -1) {
			return (-1);
		}
	}

	fread(buf, 1, 6, fstream);

	if (buf[0] == '\0' && buf[1] == 'm' && buf[2] == '\2') {
		/* if headers are ok */

		*algorithm = (unsigned int) buf[3];
		*mode = (unsigned int) buf[4];
		*bit_flag = (unsigned int) buf[5];

		if (stream_flag == FALSE) {
			fflush(fstream);
			unlock(fileno((FILE *) fstream));
			fclose(fstream);
		}
		return 0;
	} else {		/* No headers present */

		if (stream_flag == FALSE) {
			fflush(fstream);
			unlock(fileno((FILE *) fstream));
			fclose(fstream);
		} else {	/* restart the stream so we will not loose important data */
			if (fseek(fstream, 0, SEEK_SET) == -1) {
				perror("fseek");
				fprintf(stderr, _("Unable to get algorithm information. Use the --bare flag and specify the algorithm manualy.\n"));
				exit(-1);
			}
		}

		return 1;
	}
}


int write_file_head(FILE * filedes, unsigned int algorithm, unsigned int mode,
		    unsigned int bit_flag, void *salt)
{
	unsigned char *buf;

	buf = xmalloc(8);

	buf[0] = '\0';
	buf[1] = 'm';
	buf[2] = '\2';
	buf[3] = (unsigned char) algorithm;
	buf[4] = (unsigned char) mode;
	buf[5] = (unsigned char) bit_flag;

/* 6 to 9 are crc32 */

	if (fwrite(buf, 1, 6, filedes) != 6) {
		return 1;
	}
	buf = xrealloc(buf, SALT_SIZE);
	if (salt != NULL) {
		memmove(buf, salt, SALT_SIZE);
	} else {
		Bzero(buf, SALT_SIZE);
	}

	if (fwrite(buf, 1, SALT_SIZE, filedes) != SALT_SIZE) {
		return 1;
	}
	return 0;

}

#ifdef HAVE_STAT
#ifdef HAVE_UTIME
void copyDate(char *srcName, char *dstName)
{
	int retVal;
	struct stat statBuf;
	struct utimbuf uTimBuf;

	retVal = stat(srcName, &statBuf);
	if (retVal == -1)
		perror("stat");

	uTimBuf.actime = statBuf.st_atime;
	uTimBuf.modtime = statBuf.st_mtime;

	retVal = utime(dstName, &uTimBuf);
	if (retVal == -1)
		perror("utime");
}
#endif


int is_normal_file(char *filename)
{
	struct stat statBuf;

#ifdef HAVE_LSTAT		/* Do not treat symlinks as regular files */
	if (lstat(filename, &statBuf) != 0)
		return FALSE;
#else
	if (stat(filename, &statBuf) != 0)
		return FALSE;
#endif

	if (S_ISREG(statBuf.st_mode) != 0) {
		return TRUE;
	} else {
		return FALSE;
	}

}

#endif

void shandler(int signal)
{

	fprintf(stderr, _("Signal %d caught. Exiting.\n"), signal);
	cleanUp();
	exit(-1);

}

void snhandler(int signal)
{

	fprintf(stderr, _("\nSignal %d caught. Exiting.\n"), signal);
	cleanUp();
	exit(-1);

}

void cleanUp()
{

	fflush(NULL);
	if (stream_flag == FALSE && cleanDelete == TRUE)
		remove(outfile);	/* Delete the file we were writing to */

}


char **
 read_key_file(char *file, int *num)
{

	FILE *FROMF;
	char keyword[MAX_KEY_LEN], **keys = NULL;
	int x = 0;

	FROMF = fopen(file, "r");
	if (FROMF == NULL) {
		fprintf(stderr, _("Keyfile could not be opened. Ignoring it.\n"));
		return NULL;
	}
	if (read_lock(fileno((FILE *) FROMF)) == -1) {
		fprintf(stderr, _("Keyfile could not be locked. Ignoring it.\n"));
		return NULL;
	}
	while (fgets(keyword, MAX_KEY_LEN, FROMF) != NULL) {
		x++;
		keys = realloc(keys, x * sizeof(char *));
		keys[x - 1] = malloc(strlen(keyword) + 1);
/* Remove newline */
		if (keyword[strlen(keyword) - 1] == '\n')
			keyword[strlen(keyword) - 1] = '\0';
		strcpy(keys[x - 1], keyword);

	}

	*num = x;

	unlock(fileno((FILE *) FROMF));
	return keys;

}

#ifdef HAVE_GETPWUID
char *
 get_cfile(int uid, char *cfile)
{

	char *home;
	struct passwd *pwd;

	pwd = getpwuid(uid);
	home = malloc(strlen(pwd->pw_dir) + strlen(cfile) + 2);
	if (pwd != NULL) {
		strcpy(home, pwd->pw_dir);
	}
	strcat(home, "/");
	strcat(home, cfile);

	return home;

}
#endif


int ask_overwrite(char *name, char *file)
{
	char x[2];

#ifdef HAVE_SIGNAL_H
	Signal(SIGINT, snhandler);
	Signal(SIGQUIT, snhandler);
	Signal(SIGSEGV, snhandler);
	Signal(SIGPIPE, snhandler);
	Signal(SIGTERM, snhandler);
	Signal(SIGHUP, snhandler);
#endif


	fprintf(stderr, _("%s: %s already exists; do you wish to overwrite (y or n)?"), name, file);
	x[0] = getc(stdin);
	x[1] = '\0';

#ifdef HAVE_SIGNAL_H
	Signal(SIGINT, shandler);
	Signal(SIGQUIT, shandler);
	Signal(SIGSEGV, shandler);
	Signal(SIGPIPE, shandler);
	Signal(SIGTERM, shandler);
	Signal(SIGHUP, shandler);
#endif

	if (strncmp(x, "y", 1) == 0 || strncmp(x, "Y", 1) == 0) {
		return TRUE;
	} else {
		return FALSE;
	}

}

char *
 get_modes_name(int mode)
{

	char *x = xmalloc(4);

	switch (mode) {
	case CFB:
		strcpy(x, "CFB");
		return x;
		break;
	case OFB:
		strcpy(x, "OFB");
		return x;
		break;
	case CBC:
		strcpy(x, "CBC");
		return x;
		break;
	case ECB:
		strcpy(x, "ECB");
		return x;
		break;
	}
	return NULL;
}
